
#include "shared.h"

/* Interrupt handlers (in isr.s) */
extern void dummy_irq_handler(void);
extern void vbi_irq_handler(void);
extern void vbo_irq_handler(void);
extern void hbi_irq_handler(void);
extern void tm0_irq_handler(void);
extern void tm1_irq_handler(void);

/* Place vector table at end of low work RAM */
#define VBR_ADDR    0x002FFC00

uint32 frame_count;
uint32 irq_mask;
uint32 old_mask;
uint32 old_sr;
uint32 old_vbr;

void irq_init(void)
{
    int i;
    uint32 *vector = (uint32 *)VBR_ADDR;

    /* Point all vectors to empty function */
    for(i = 0; i < 0x100; i++)
        vector[i] = (uint32)&dummy_irq_handler;

    /* Point useful vectors to our routines */
    vector[0x40] = (uint32)&vbi_irq_handler;
    vector[0x41] = (uint32)&vbo_irq_handler;
    vector[0x42] = (uint32)&hbi_irq_handler;
    vector[0x43] = (uint32)&tm0_irq_handler;
    vector[0x44] = (uint32)&tm1_irq_handler;

    /* Save old settings */
    old_mask = SCU_IRQ_MASK;
    old_vbr = get_vbr();
    old_sr = get_sr();

    /* Load new settings */
    frame_count = 0;
    SCU_IRQ_MASK = irq_mask = 0x0000BFFF;
    set_vbr(VBR_ADDR);
    set_sr(old_sr | 0x000000F0);
}

void irq_shutdown(void)
{
    /* Restore old settings */
    SCU_IRQ_MASK = old_mask;
    set_vbr(old_vbr);
    set_sr(old_sr);
}
