#ifndef _IRQ_H_
#define _IRQ_H_

/* SCU IRQ mask and status flags */
#define SCU_IRQ_VBLANK_IN   0x00000001
#define SCU_IRQ_VBLANK_OUT  0x00000002
#define SCU_IRQ_HBLANK_IN   0x00000004
#define SCU_IRQ_TIMER0      0x00000008
#define SCU_IRQ_TIMER1      0x00000010

/* SCU registers */
#define SCU_TIMER0_COMPARE  (*(vuint32 *)0x25FE0090)
#define SCU_TIMER1_SET      (*(vuint32 *)0x25FE0094)
#define SCU_TIMER1_CONTROL  (*(vuint32 *)0x25FE0098)
#define SCU_IRQ_MASK        (*(vuint32 *)0x25FE00A0)
#define SCU_IRQ_STATUS      (*(vuint32 *)0x25FE00A4)

/* Global data */
extern uint32 frame_count;
extern uint32 irq_mask;
extern uint32 old_mask;
extern uint32 old_sr;
extern uint32 old_vbr;

/* Function prototypes */
void irq_init(void);
void irq_shutdown(void);

/* Inline functions to access SH-2 registers */
static inline void set_sr(uint32 sr)
{
    asm volatile ("ldc\t%0,sr"::"r"(sr));
}

static inline uint32 get_sr(void)
{
    uint32 sr;
    asm volatile ("stc\tsr,%0":"=r"(sr));
    return sr;
}

static inline void set_vbr(uint32 vbr)
{
    asm volatile ("ldc\t%0,vbr"::"r"(vbr));
}

static inline uint32 get_vbr(void)
{
    uint32 vbr;
    asm volatile ("stc\tvbr,%0":"=r"(vbr));
    return vbr;
}

static inline void sleep(void)
{
    asm volatile ("sleep");
}

static inline void nop(void)
{
    asm volatile ("nop");
}

#endif /* _IRQ_H_ */
