
                        .section .text

                        .extern  _irq_mask
                        .extern  _frame_count

                        .global  _dummy_irq_handler
                        .global  _vbi_irq_handler
                        .global  _vbo_irq_handler
                        .global  _hbi_irq_handler
                        .global  _tm0_irq_handler
                        .global  _tm1_irq_handler

!----------------------------------------------------------------------------
! Dummy interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_dummy_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Set IRQ mask to acknowledge interrupt
                        mov.l   scu_irq_mask, r0
                        mov.l   irq_mask_ptr, r1
                        mov.l   @r1, r1
                        mov.l   r1, @r0

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0
                        rte
                        nop

!----------------------------------------------------------------------------
! V-Blank IN interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_vbi_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Bump frame count
                        mov.l   frame_count_ptr, r0
                        mov.l   @r0, r1
                        add     #0x01, r1
                        mov.l   r1, @r0

                        ! Set IRQ mask to acknowledge interrupt
                        mov.l   scu_irq_mask, r0
                        mov.l   irq_mask_ptr, r1
                        mov.l   @r1, r1
                        mov.l   r1, @r0

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0
                        rte
                        nop

!----------------------------------------------------------------------------
! V-Blank OUT interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_vbo_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Set timer 1 to run on all lines
                        mov.l   scu_tm1_ctrl, r0
                        mov.l   ctrl_data_vbo, r1
                        mov.l   r1, @r0

                        ! Set IRQ mask to acknowledge interrupt
                        ! and enable HBI interrupt
                        mov.l   irq_mask_ptr, r0
                        mov.l   @r0, r1                 
                        mov.l   irq_mask_and, r0
                        and     r0, r1                  
                        mov.l   irq_mask_ptr, r0
                        mov.l   r1, @r0                 
                        mov.l   scu_irq_mask, r0
                        mov.l   r1, @r0

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0
                        rte
                        nop

!----------------------------------------------------------------------------
! H-Blank IN interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_hbi_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Change screen mode
                        mov.l   tvmd_addr, r0
                        mov.w   tvmd_data_hbi, r1
                        mov.w   r1, @r0

                        ! Set IRQ mask to acknowledge interrupt
                        mov.l   scu_irq_mask, r0
                        mov.l   irq_mask_ptr, r1
                        mov.l   @r1, r1
                        mov.l   r1, @r0

                        ! Clear SR so a timer 1 interrupt can occur,
                        ! and sleep to reduce flicker when it does
                        ldc     r0, sr
                        mov.l   irq_mask_sr, r1
                        and     r1, r0
                        stc     sr, r0
                        sleep

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0
                        rte
                        nop

!----------------------------------------------------------------------------
! Timer 0 interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_tm0_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Turn off timer 1, which still executes on this
                        ! line regardless of being turned off
                        mov.l   scu_tm1_ctrl, r0
                        mov.l   ctrl_data_tm1, r1
                        mov.l   r1, @r0

                        ! Set IRQ mask to acknowledge interrupt
                        ! and disable HBI interrupt for rest of frame
                        mov.l   irq_mask_ptr, r0
                        mov.l   @r0, r1                 
                        mov.l   irq_mask_or, r0
                        or      r0, r1                  
                        mov.l   irq_mask_ptr, r0
                        mov.l   r1, @r0                 
                        mov.l   scu_irq_mask, r0
                        mov.l   r1, @r0

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0

                        rte
                        nop

!----------------------------------------------------------------------------
! Timer 1 interrupt handler
!----------------------------------------------------------------------------
                        .align 4
_tm1_irq_handler:
                        mov.l   r0, @-r15
                        mov.l   r1, @-r15

                        ! Change screen mode
                        mov.l   tvmd_addr, r0
                        mov.w   tvmd_data_tm1, r1
                        mov.w   r1, @r0

                        ! Set IRQ mask to acknowledge interrupt
                        mov.l   scu_irq_mask, r0
                        mov.l   irq_mask_ptr, r1
                        mov.l   @r1, r1
                        mov.l   r1, @r0

                        mov.l   @r15+, r1
                        mov.l   @r15+, r0
                        rte
                        nop

!----------------------------------------------------------------------------

                        .align  4
scu_irq_mask:           .long   0x25FE00A0
scu_tm1_ctrl:           .long   0x25FE0098
irq_mask_ptr:           .long   _irq_mask
frame_count_ptr:        .long   _frame_count

ctrl_data_tm1:          .long   0x00000000      ! Timer 1 disabled
ctrl_data_vbo:          .long   0x00000001      ! Timer 1 works on all lines
irq_mask_and:           .long   0xFFFFFFFB      ! Enable HBI
irq_mask_or:            .long   0x00000004      ! Disable HBI
irq_mask_sr:            .long   0xFFFFFF0F      ! Enable interrupts
tvmd_addr:              .long   0x25F80000
tvmd_data_hbi:          .word   0x8000          ! 320 pixel mode
tvmd_data_tm1:          .word   0x8002          ! 640 pixel mode

